!
! EMT potential force and potential (and stress term) calculation
!  for Cu, Ag, Au, Ni, Pd, Pt
!
! K.W.Jacobsen, P.Stoltze and J.K.Norskov
!  Surface Science 366 (1996) 394-402
!  "A semi-empirical effective medium theory for metals and alloys"
!  New bookkeep algorithm
!
! When 'gamma_search' is true, gamma_1 and gamma_2 are searched.
! (Not necessary when g_1 and g_2 are known)
! Use when the cutoff radius is changed, with FCC configration file.
! Change tolerance when an error occurs (or maybe search range...)
!
!   Feb. 24 (2006)
!   Stomic stress calculation is implemented
!   Oct. 5, 2010
!   A bug in stress calculation for a small cell is fixed

      module emt_JSN_param
      double precision :: emte0, emts0, emtv0, emteta2, emtkap, emtlam, emtn0
      double precision :: emtbet, emtrc, emta, emtgam1, emtgam2
      logical :: gamma_search = .false. ! gamma_1,gamma_2 search mode
      double precision :: gs_tol1=1.0d-5 ! Tolerance for gamma_1 search
      double precision :: gs_tol2=1.0d-4 ! Tolerance for gamma_2 search
      logical :: emt_JSN_setup_done = .false.
      end module

      subroutine emt_JSN_SS1996
      use emt_JSN_param
      use variables
      use constants
      use bookkeep_param
      use debug_variables
      implicit double precision (a-h,o-z)
      double precision :: emtsig1(n), emtsig2(n), emts(n)
      double precision :: ki,kif
      integer :: istat

!
      istat=1

! Parameter set
      if (.not.emt_JSN_setup_done) then
         emt_JSN_setup_done = .true.
         print *,'EMT (JSN) setup'
         if ((asp(1)=='Cu').or.(asp(1)=='cu')) then
            emte0   = -3.51d0  *ev
            emts0   =  2.67d0  *bohr
            emtv0   =  2.476d0 *ev
            emteta2 =  1.652d0 /bohr
            emtkap  =  2.740d0 /bohr
            emtlam  =  1.906d0 /bohr
            emtn0   =  0.00910 /bohr**3
            emtgam1 = 1.03326d0; emtgam2 = 1.04259d0 ! Ohkura
!           emtgam1 = 1.0246d0; emtgam2 = 1.0345d0 ! Ohkura
         else if ((asp(1)=='Ag').or.(asp(1)=='ag')) then
            emte0   = -2.96d0  *ev
            emts0   =  3.01d0  *bohr
            emtv0   =  2.132d0 *ev
            emteta2 =  1.652d0 /bohr
            emtkap  =  2.790d0 /bohr
            emtlam  =  1.892d0 /bohr
            emtn0   =  0.00547 /bohr**3
            emtgam1 = 0.97947d0; emtgam2 = 0.98660d0
         else if ((asp(1)=='Au').or.(asp(1)=='au')) then
            emte0   = -3.80d0  *ev
            emts0   =  3.00d0  *bohr
            emtv0   =  2.321d0 *ev
            emteta2 =  1.674d0 /bohr
            emtkap  =  2.873d0 /bohr
            emtlam  =  2.182d0 /bohr
            emtn0   =  0.00703 /bohr**3
            emtgam1 = 0.97159d0; emtgam2 = 0.97716d0
         else if ((asp(1)=='Ni').or.(asp(1)=='ni')) then
            emte0   = -4.44d0  *ev
            emts0   =  2.60d0  *bohr
            emtv0   =  3.673d0 *ev
            emteta2 =  1.669d0 /bohr
            emtkap  =  2.757d0 /bohr
            emtlam  =  1.948d0 /bohr
            emtn0   =  0.01030 /bohr**3
            emtgam1 = 1.02440d0; emtgam2 = 1.03558d0
         else if ((asp(1)=='Pd').or.(asp(1)=='pd')) then
            emte0   = -3.90d0  *ev
            emts0   =  2.87d0  *bohr
            emtv0   =  2.773d0 *ev
            emteta2 =  1.818d0 /bohr
            emtkap  =  3.107d0 /bohr
            emtlam  =  2.155d0 /bohr
            emtn0   =  0.00688 /bohr**3
            emtgam1 = 0.99946d0; emtgam2 = 1.00353d0
         else if ((asp(1)=='Pt').or.(asp(1)=='pt')) then
            emte0   = -5.85d0  *ev
            emts0   =  2.90d0  *bohr
            emtv0   =  4.067d0 *ev
            emteta2 =  1.812d0 /bohr
            emtkap  =  3.145d0 /bohr
            emtlam  =  2.192d0 /bohr
            emtn0   =  0.00802 /bohr**3
            emtgam1 = 1.02600d0; emtgam2 = 1.02783d0
         else
            print *,'emt_JSN_SS1996 is prepared for Cu, Ag, Au, Ni, Pd and Pt'
            print *, asp(1),' is not supported'
            stop
         endif
         emtbet  = (16.0d0*pi/3.0d0)**(1.0d0/3.0d0)/sqrt(2.0d0)
         emtrc=(sqrt(6.0)+sqrt(7.0))*emts0*emtbet/2.0d0
         emta    = 20.0d0/bohr
      endif
      rc = emtrc*1.1d0
      rc2 = rc**2

!   Virial term reset
      virx=0.0d0; viry=0.0d0; virz=0.0d0
      dmat=0.0d0
      satom=0.0d0               ! Atomic stress

      if (gamma_search) then
         print *,'Gamma search mode'
         emtgam1=0.9d0; emtgam2=-1.0d0
      endif
 1111 continue

!   First Loop: Chg dns calc
      emtsig1=0.0d0
      emtsig2=0.0d0
      kns=0
      do 4200 k=1,n
         kn=knum(k)
         if (kn.eq.0) go to 4200
         do 4300 jk=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(k)-rx(j)
            yy=ry(k)-ry(j)
            zz=rz(k)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            if (istat == 1) then
               r2=xx**2+yy**2+zz**2
               if (r2.lt.rc2) then
                  r=sqrt(r2)
                  emtthe=1.0d0/(1.0d0+exp(emta*(r-emtrc)))
                  dxx1=exp(-emteta2*(r-emtbet*emts0))*emtthe
                  dyy1=exp(-emtkap/emtbet*(r-emtbet*emts0))*emtthe
                  emtsig1(k)=emtsig1(k)+dxx1
                  emtsig2(k)=emtsig2(k)+dyy1
                  if (j/=k) then
                     emtsig1(j)=emtsig1(j)+dxx1
                     emtsig2(j)=emtsig2(j)+dyy1
                  endif
               endif
            endif
 4300    continue
 4200 continue
      
      emtsig1=emtsig1/emtgam1
      emtsig2=emtsig2/emtgam2

      emts=emts0-dlog(emtsig1/12.0d0)/(emtbet*emteta2)

!     Calculation of potential energy
      enpoteach=emte0*(1.0d0+emtlam*(emts-emts0))*exp(-emtlam*(emts-emts0)) &
      -emtv0/2.0d0*emtsig2+6.0d0*emtv0*exp(-emtkap*(emts-emts0))

!      ec=0.0d0; eas=0.0d0
!      do i=1,n
!         ec=ec+emte0*(1.0d0+emtlam*(emts(i)-emts0))*exp(-emtlam*(emts(i)-emts0))
!         eas=eas-emtv0/2.0d0*emtsig2(i)+6.0d0*emtv0*exp(-emtkap*(emts(i)-emts0))
!      enddo
      if (gamma_search) then
         ec=0.0d0; eas=0.0d0
         do i=1,n
            ec=ec+emte0*(1.0d0+emtlam*(emts(i)-emts0))*exp(-emtlam*(emts(i)-emts0))
            eas=eas-emtv0/2.0d0*emtsig2(i)+6.0d0*emtv0*exp(-emtkap*(emts(i)-emts0))
         enddo
         if (emtgam2<0.0) then  ! g_2 is negative until g_1 is determined
            if (ec/ev/n-emte0/ev<gs_tol1) then
               print *,'gamma_1=',emtgam1
               emtgam2=0.9d0
               go to 1111
            else
               emtgam1=emtgam1+0.00001
               if (emtgam1>1.2) then 
                  print *,'gamma1 error'; stop
               endif
               go to 1111
            endif
         else
            if (abs(eas/ev/n)<gs_tol2) then
               print *,'gamma_2=',emtgam2
               print *,'Ec=',ec/ev/n,' Eas=',eas/ev/n
               print *,'rc=',emtrc/ang
               stop
            else
               emtgam2=emtgam2+0.00001
               if (emtgam2>1.2) then
                  print *,'gamma2 error'; stop
               endif
               go to 1111
            endif
         endif
      endif

!   Second Loop: Force calc
      fx(1:n)=0.0d0
      fy(1:n)=0.0d0
      fz(1:n)=0.0d0
!      const_n=-emtn0*emteta*emteta2*dexp(emteta2*emtbet*emts0) &
!      /12.0d0/gamma1/(emteta+emteta1)
      
!      const_v=emtalp*emtn0*emteta*dexp(emteta*emts0)/(6.0d0*emtbet*gamma2)

      kns=0
      do 4500 i=1,n
         kn=knum(i)
         if (kn.eq.0) go to 4500
         do 4600 jk=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(i)-rx(j)
            yy=ry(i)-ry(j)
            zz=rz(i)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            if (istat == 1) then
               r2=xx**2+yy**2+zz**2
               if (r2.lt.rc2) then
                  r=sqrt(r2)
!                  vij=-const_v*emtbet/emteta*dexp(-emteta*r/emtbet)
!                  enpoteach(i)=enpoteach(i)+vij/2.0d0
!                  vijsum=vijsum+vij/2.0d0
!                  if (j/=i) then
!                     enpoteach(j)=enpoteach(j)+vij/2.0d0
!                     vijsum=vijsum+vij/2.0d0
!                  endif
                  
! Ec term
                  dec_i=-emte0*emtlam**2*(emts(i)-emts0)*exp(-emtlam*(emts(i)-emts0))
                  dec_j=-emte0*emtlam**2*(emts(j)-emts0)*exp(-emtlam*(emts(j)-emts0))

                  theta=1.0d0/(1.0d0+exp(emta*(r-emtrc)))
                  dtheta=emta*theta*(theta-1.0d0)
                  bs0=emtbet*emts0
                  ecconst=exp(-emteta2*(r-bs0))*(-emteta2*theta+dtheta) &
                  /(emtbet*emteta2*emtgam1)
                  dec=(dec_i/emtsig1(i)+dec_j/emtsig1(j))*ecconst
! Eas term
                  deas1=emtv0/emtgam2*exp(-emtkap/emtbet*(r-bs0)) &
                  *(-emtkap/emtbet*theta+dtheta)

                  des2const=-6.0d0*emtv0*emtkap/(emtbet*emteta2*emtgam1)
                  des2_i=exp(-emtkap*(emts(i)-emts0))/emtsig1(i)
                  des2_j=exp(-emtkap*(emts(j)-emts0))/emtsig1(j)
                  deas2=(des2_i+des2_j)*exp(-emteta2*(r-bs0)) &
                  *(-emteta2*theta+dtheta)*des2const

                  f=dec+deas1+deas2

                  ki=f/r
                  if (j==i) ki=ki/2.0
                  kif=ki/2.0d0
                  dxx1=ki*xx
                  dyy1=ki*yy
                  dzz1=ki*zz
                  fx(i)=fx(i)+dxx1
                  fy(i)=fy(i)+dyy1
                  fz(i)=fz(i)+dzz1
                  fx(j)=fx(j)-dxx1
                  fy(j)=fy(j)-dyy1
                  fz(j)=fz(j)-dzz1
                  satom(i,1,1)=satom(i,1,1)-kif*xx*xx
                  satom(i,1,2)=satom(i,1,2)-kif*yy*xx
                  satom(i,2,2)=satom(i,2,2)-kif*yy*yy
                  satom(i,1,3)=satom(i,1,3)-kif*zz*xx
                  satom(i,2,3)=satom(i,2,3)-kif*zz*yy
                  satom(i,3,3)=satom(i,3,3)-kif*zz*zz
                  satom(j,1,1)=satom(j,1,1)-kif*xx*xx
                  satom(j,1,2)=satom(j,1,2)-kif*yy*xx
                  satom(j,2,2)=satom(j,2,2)-kif*yy*yy
                  satom(j,1,3)=satom(j,1,3)-kif*zz*xx
                  satom(j,2,3)=satom(j,2,3)-kif*zz*yy
                  satom(j,3,3)=satom(j,3,3)-kif*zz*zz
               endif
            endif
 4600    continue
 4500 continue

!---Etot---
      enpot=0.0d0
      do i=1,n
         enpot=enpot+enpoteach(i)
      enddo

!!     Atomic stress
      do i=1,n
         satom(i,2,1)=satom(i,1,2)
         satom(i,3,1)=satom(i,1,3)
         satom(i,3,2)=satom(i,2,3)
      enddo
      dmat=0.0d0
! The sign of dmat is opposite to stress tensor's (dmat is the "virial" tensor)
      do i=1,n
         dmat(1:3,1:3)=dmat(1:3,1:3)-satom(i,1:3,1:3)
      enddo
      virx=dmat(1,1); viry=dmat(2,2); virz=dmat(3,3)

      satom=satom*dble(n)/v

!      write (10,*) 'EMT force calculation end.'
      end


